@@ -135,6 +135,9 @@ _cairo_boilerplate_gl_create_surface (const char		*name,
     gltc->ctx = ctx;
     gltc->device = cairo_glx_device_create (dpy, ctx);
 
+    if (mode == CAIRO_BOILERPLATE_MODE_PERF)
+	cairo_gl_device_set_thread_aware(gltc->device, FALSE);
+
     gltc->surface = surface = cairo_gl_surface_create (gltc->device,
 						       content, width, height);
     if (cairo_surface_status (surface))
