@@ -113,7 +113,12 @@ cairo_boilerplate_format_from_content (cairo_content_t content);
 
 typedef enum {
     CAIRO_BOILERPLATE_MODE_TEST,
-    CAIRO_BOILERPLATE_MODE_PERF
+    CAIRO_BOILERPLATE_MODE_PERF,
+
+    /* This will allow running performance test with threads. The
+     * GL backend is very slow on some drivers when run with thread
+     * awareness turned on. */
+    CAIRO_BOILERPLATE_MODE_PERF_THREADS,
 } cairo_boilerplate_mode_t;
 
 typedef cairo_surface_t *
